/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.importer;

// Kelp imports
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.ValidationUtil;
import org.enhydra.kelp.common.xmlc.MapPanel;
import org.enhydra.kelp.common.xmlc.MapDialog;

// Standard imports
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.beans.*;
import java.io.File;
import java.awt.event.*;
import java.util.ResourceBundle;

/**
 * Panel for entering the default options: project name, package
 * and destination.
 *
 * @author Paul Mahar
 */
public class ImportMapPanel extends ImporterPanel {
    static ResourceBundle           res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    transient private MapPanel      mapPanel;
    transient private JPanel        panelPack;
    transient private JTextField    textPack;

    //
    transient private JPanel        panelEdit;
    transient private GridBagLayout layoutMain;
    transient private GridBagLayout layoutPack;
    transient private TitledBorder  borderPack;
    transient private JButton       buttonEdit;
    transient private GridBagLayout layoutEdit;

    /**
     * Create default option panel 1 for entering the
     * following default options: project name, package and
     * destination.
     */
    public ImportMapPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Read project name, package and destination values from the
     * option set into the swing controls.
     *
     * @exception GeneratorException
     * Thrown if unable to update Swing controls with option set values.
     */
    public void readOptions() {
        String pack = null;
        String map[][] = new String[0][2];

        pack = getPaths().getPackage();
        if (pack != null) {
            textPack.setText(pack);
        }
        map = getPaths().getPackageMap();
        mapPanel.setMap(map);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    public void activated() {
        initPacks();
    }

    /**
     * Write project name, package and destination values from the
     * swing controls into the option set.
     *
     * @exception GeneratorException
     * Thrown if unable to update option set from Swing control values.
     */
    public void writeOptions() throws ValidationException {
        String     pack = new String();
        String[][] map = new String[0][2];

        if (getPaths() == null) {
            throw new ValidationException(res.getString("ImportPaths_not_set"));
        }
        pack = textPack.getText().trim();
        getPaths().setPackage(pack);
        map = mapPanel.getMap();
        getPaths().setPackageMap(map);
    }

    /**
     * Validate project name, package and destination values
     * in the swing controls.
     *
     * @exception ValidationException
     * Thrown if swing control values are not valid for the
     * current option set.
     */
    public void validateOptions() throws ValidationException {
        String value = new String();

        // package
        value = textPack.getText().trim();
        if (!ValidationUtil.isJavaPackage(value)) {
            throw new ValidationException(res.getString("Root_invalid_java"));
        }
    }

    /**
     * Get the title to use on the current page.
     *
     * @return
     * A string to place at the top of a CodeGen wizard panel.
     */
    public String getPageTitle() {
        return res.getString("Package_details");
    }

    /**
     * Get the instructions for entering option values for the
     * current page.
     *
     * @return
     * A string to place below the page title.
     */
    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append(res.getString("Enter_package_details"));
        buf.append(res.getString("All_the_application"));
        buf.append(res.getString("defined_at_or_below"));
        buf.append(res.getString("The_package_map_is"));
        buf.append(res.getString("for_source_files"));
        return buf.toString();
    }

    // /
    // /

    /**
     * Method declaration
     *
     *
     * @exception Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
        textPack = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        panelPack = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        mapPanel = (MapPanel) Beans.instantiate(getClass().getClassLoader(),
                                                MapPanel.class.getName());
        layoutPack =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelEdit = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        buttonEdit = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        layoutEdit =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelPack.setLayout(layoutPack);
        buttonEdit.setText(res.getString("buttonEdit_Text"));
        buttonEdit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                editPackageMap();
            }

        });
        panelEdit.setLayout(layoutEdit);
        panelEdit.add(buttonEdit,
                      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.BOTH,
                                             new Insets(5, 5, 5, 5), 0, 0));
        panelPack.add(textPack,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(5, 5, 5, 5), 0, 0));
        this.setLayout(layoutMain);
        this.add(panelPack,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(mapPanel,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
        this.add(panelEdit,
                 new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 0, 0));
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
        Border border = BorderFactory.createEtchedBorder();

        borderPack = BorderFactory.createTitledBorder(border);
        borderPack.setTitle(res.getString("borderPack_Title"));
        panelPack.setBorder(borderPack);
        textPack.setText(new String());
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void initPacks() {
        String     pack = null;
        String[][] map = new String[0][2];

        if (textPack.getText().trim().length() == 0) {
            pack = getPaths().getDefaultPackage();
            if (pack != null) {
                try {
                    getPaths().setPackage(pack);
                } catch (ValidationException e) {
                    e.printStackTrace();
                }
                textPack.setText(pack);
            }
        }
        if (mapPanel.getMap().length == 0) {
            getPaths().initPackageMap(false);
            map = getPaths().getPackageMap();
            mapPanel.setMap(map);
        }
    }

    private void editPackageMap() {
        MapDialog dialog =
            new MapDialog(((JDialog) this.getTopLevelAncestor()),
                          res.getString("Edit_Package_Map"), true);

        getPaths().getProject().setPackageMap(mapPanel.getMap());
        dialog.setProject(getPaths().getProject());
        dialog.show();
        mapPanel.setMap(getPaths().getProject().getPackageMap());
        getPaths().setPackageMap(mapPanel.getMap());
    }

}
