/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// Toolbox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.event.ProgressListener;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ArchiveTool;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.WarPlan;

// Kelp imports
import org.enhydra.kelp.KelpInfo;
import org.enhydra.kelp.common.AbstractEchoBuilder;
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;

// Standard imports
import java.awt.Component;
import java.awt.Window;
import java.io.File;
import java.util.StringTokenizer;
import java.util.ResourceBundle;
import javax.swing.JOptionPane;

//
public class DeployBuilder extends AbstractEchoBuilder {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores

    //
    private OtterNode[]   inputNodes = null;
    private Window        owner = null;
    private boolean       doneDialog = false;

    public static void preBuild(Object source, OtterProject project,
                                WriteListener listener) {
        PathHandle path = null;
        WriteEvent event = null;

        path = PathHandle.createPathHandle(project.getDeployRootPath());
        if (!path.isDirectory()) {
            path.getFile().mkdirs();
            event = new WriteEvent(source, WriteEvent.OUTPUT,
                                   "Created directory: " + path.getPath());
            listener.onWrite(event);
        }
    }

    public DeployBuilder() {
        super(null);
    }

    public DeployBuilder(WriteListener listener) {
        super(listener);
    }

    public DeployBuilder(WriteListener listener, boolean buffer) {
        super(listener, buffer);
    }

    public void setInputNodes(OtterNode[] n) {
        inputNodes = n;
    }

    public OtterNode[] getInputNodes() {
        return inputNodes;
    }

    public void buildInput() {
        InputBuilder builder = null;

        builder = new InputBuilder(getWriteListener());
        if (getInputNodes() != null) {
            builder.setNodes(getInputNodes());
        }
        invoke(builder);
    }

    public void buildContent() {
        ContentBuilder builder = null;

        builder = new ContentBuilder(getWriteListener());
        invoke(builder);
    }

    public void buildArchives() {
        ArchiveBuilder builder = null;

        builder = new ArchiveBuilder(getWriteListener());
        invoke(builder);
    }

    public void buildRun() {
        RunBuilder builder = null;

        builder = new RunBuilder(getWriteListener());
        invoke(builder);
    }

    /**
     * Make content, policy file, process templates, configure project for
     * running enhydra and create archive.
     */
    public void buildImpl() {
        Exception ex = null;

        try {
            echo(new String());
            refreshProgress(5, res.getString("Deployment_starting"));
            echo(res.getString("_Deployment_starting"));
            if (isFresh()) {
                refreshProgress(5, res.getString("Processing_input"));
                echo(res.getString("Processing_input"));
                buildInput();
            }
            if (isFresh()) {
                refreshProgress(30, res.getString("Deploying_content"));
                echo(res.getString("Deploying_content"));
                buildContent();
            }
            if (isFresh()) {
                refreshProgress(60, "Creating archives");
                echo("Creating archives");
                buildArchives();
            }
            if (isFresh()) {
                if (getProject().isDeployRun()) {
                  refreshProgress(90, "Configuring project run");
                  echo("Configuring project run");
                  buildRun();
                } else {
                  refreshProgress(90, "");
                }
            }
            echo("--- Kelp Deployer finished --- ");
        } catch (Exception e) {
            ex = e;
            echo(ex);
        }
        refreshProgress(100, new String());
        if (isDoneDialog()) {
            StringBuffer buf = new StringBuffer();

            if (ex == null) {
                buf.append("Deployment Complete");
            } else {
                buf.append("Exception during deployment:\n");
                if (ex.getMessage() == null) {
                    buf.append(ex.toString());
                } else {
                    buf.append(ex.getMessage());
                }
                buf.append("\n");
                buf.append("See messages for more information.\n");
            }
            JOptionPane.showMessageDialog(getOwner(), buf.toString());
        }
    }

    //

    protected void setOwner(Window w) {
        owner = w;
    }

    protected Window getOwner() {
        return owner;
    }

    protected boolean isDoneDialog() {
        boolean canDone = doneDialog;

        if (getOwner() == null) {
            canDone = false;
        }
        return canDone;
    }

    protected void setDoneDialog(boolean d) {
        doneDialog = d;
    }

    //
    //

    private void invoke(AbstractEchoBuilder builder) {
        ProgressListener[] pl = getProgressListeners();

        if (pl != null) {
            for (int i = 0; i < pl.length; i++) {
                builder.addProgressListener(pl[i]);
            }
        }
        builder.setProject(getProject());
        builder.setEcho(isEcho());
        builder.buildInCurrentThread();
        if (pl != null) {
            for (int i = 0; i < pl.length; i++) {
                builder.removeProgressListener(pl[i]);
            }
        }
    }

}
