/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// Kelp imports
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.AddinInnerPanel;

// ToolBox imports
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.ButtonPanel;
import org.enhydra.tool.common.ToolException;

// Standard imports
import java.awt.*;
import java.lang.ref.WeakReference;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

//
public class DeployInnerPanel extends AddinInnerPanel {
    private int               showIndex = 0;
    private JTabbedPane       tab = null;
    private BorderLayout      layoutMain = null;
    private GeneralPanel      generalPanel = null;
    private InputPanel        inputPanel = null;
    private ContentPanel      contentPanel = null;
    private ArchivePanel      archivePanel = null;
    private RunPanel          runPanel = null;
    private TabChangeListener tabListener = null;
    private WeakReference     backRef = null;
    private WeakReference     nextRef = null;

    public DeployInnerPanel() {
        super();
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // override AddinInnerPanel
    public void preShow() {
        Component back = getBack();

        if (back != null) {
            back.setEnabled(false);
        }
    }

    // override AddinInnerPanel
    public void read(OtterNode node) {
        super.read(node);
        DeployStep[] steps = new DeployStep[0];

        steps = getSteps();
        for (int i = 0; i < steps.length; i++) {
            steps[i].read(getProject());
        }
    }

    // override AddinInnerPanel
    public void write(OtterNode node) throws DataValidationException {
        super.write(node);
        DeployStep[] steps = new DeployStep[0];

        steps = getSteps();
        for (int i = 0; i < steps.length; i++) {
            steps[i].write(getProject());
        }
    }

    // override AddinInnerPanel
    public void initPreferredSize() {
        Dimension d = new Dimension();

        d.height = 425;
        d.width = 600;
        setPreferredSize(d);
    }

    // override AddinInnerPanel
    public void clearAll() {
        super.clearAll();
        DeployStep[] steps = new DeployStep[0];

        tab.removeChangeListener(tabListener);
        if (backRef != null) {
            backRef.clear();
        }
        if (nextRef != null) {
            nextRef.clear();
        }
        steps = getSteps();
        for (int i = 0; i < steps.length; i++) {
            steps[i].clearAll();
        }
        backRef = null;
        nextRef = null;
        tabListener = null;
        generalPanel = null;
        inputPanel = null;
        contentPanel = null;
        archivePanel = null;
        runPanel = null;
    }

    //
    private void refresh() {
        Component back = null;
        Component next = null;
        boolean   backEnable = true;
        boolean   nextEnable = true;

        back = getBack();
        next = getNext();
        int index = tab.getSelectedIndex();

        if (index == 0) {
            backEnable = false;
        } else if (index == (tab.getTabCount() - 1)) {
            nextEnable = false;
        }
        if (back != null) {
            back.setEnabled(backEnable);
        }
        if (next != null) {
            next.setEnabled(nextEnable);
        }
        getSelectedStep().refresh();
    }

    private Component getNext() {
        Component comp = null;

        if (nextRef == null) {
            comp = ButtonPanel.findButton(getTopLevelAncestor(),
                                          ButtonPanel.COMMAND_NEXT);
            if (comp != null) {
                nextRef = new WeakReference(comp);
            }
        } else {
            comp = (Component) nextRef.get();
        }
        return comp;
    }

    private Component getBack() {
        Component comp = null;

        if (backRef == null) {
            comp = ButtonPanel.findButton(getTopLevelAncestor(),
                                          ButtonPanel.COMMAND_BACK);
            if (comp != null) {
                backRef = new WeakReference(comp);
            }
        } else {
            comp = (Component) backRef.get();
        }
        return comp;
    }

    protected void back() {
        int index = tab.getSelectedIndex();

        if (index > 0) {
            showIndex = --index;
            tab.setSelectedIndex(showIndex);
        }
        refresh();
    }

    protected boolean next() {
        int        index = tab.getSelectedIndex();
        boolean    valid = true;
        DeployStep step = getSelectedStep();

        if (index < tab.getTabCount()) {
            try {
                step.validateData();
                if (getProject() != null) {
                    step.write(getProject());
                    step.build(getProject(), getWriteListener());
                }
                showIndex = ++index;
                tab.setSelectedIndex(showIndex);
            } catch (DataValidationException e) {
                valid = false;
                JOptionPane.showMessageDialog(this, e.getValidationMessage());
            } catch (ToolException e) {
                valid = false;
                JOptionPane.showMessageDialog(this, e.getMessage());
                e.printStackTrace(System.err);
            }
        }
        refresh();
        return valid;
    }

    private WriteListener getWriteListener() {
        return generalPanel.getWriteListener();
    }

    protected void deploy(CoreDeployTool tool) {
        if (getProject() != null) {
            DeployBuilder builder = null;

            try {
                WriteEvent clear;

                clear = new WriteEvent(this, WriteEvent.CLEAR, new String());
                trySave();
                builder = new DeployBuilder(getWriteListener());
                builder.setProject(getProject());
                builder.setEcho(true);
                builder.addProgressListener(tool.getProgressMeter());
                tool.getProgressMeter().addCancelListener(builder);
                tool.openProgress();
                getWriteListener().onWrite(clear);
                if (tab.getSelectedIndex() > 0) {
                    builder.setOwner(getWindow());
                    builder.setDoneDialog(true);
                }
                builder.build();
            } catch (DataValidationException e) {
                JOptionPane.showMessageDialog(this, e.getValidationMessage());
            } catch (ToolException e) {
                JOptionPane.showMessageDialog(this, e.getMessage());
                e.printStackTrace(System.err);
            }
        }
    }

    // override InnerPanel
    protected Component[] getFirstFocusComponents() {
        Component[] comps = new Component[0];
        Component[] genComps = new Component[0];

        genComps = generalPanel.getFirstFocusComponents();
        comps = new Component[genComps.length + 1];
        comps[0] = tab;
        for (int i = 0; i < genComps.length; i++) {
            comps[i + 1] = genComps[i];
        }
        return comps;
    }

    // override InnerPanel
    public void save() {
        if (getProject() == null) {
            System.err.println("DeployInnerPanel.save() - project null");
        } else {
            try {
                trySave();
            } catch (DataValidationException e) {
                System.err.println(e.getValidationMessage());
            } catch (ToolException e) {
                System.err.println(e.getMessage());
            }
        }
    }

    private void trySave() throws DataValidationException, ToolException {
        try {
            write(getProject());
        } catch (DataValidationException e) {
            buildSteps();
        }
    }

    private void buildSteps() throws DataValidationException, ToolException {
        DeployStep[] steps = new DeployStep[0];

        steps = getSteps();
        for (int i = 0; i < steps.length; i++) {
            steps[i].build(getProject(), getWriteListener());
        }
    }

    //
    private DeployStep getSelectedStep() {
        DeployStep[] steps = new DeployStep[5];

        steps = getSteps();
        return steps[tab.getSelectedIndex()];
    }

    private DeployStep[] getSteps() {
        DeployStep[] steps = new DeployStep[5];

        steps[0] = generalPanel;
        steps[1] = inputPanel;
        steps[2] = contentPanel;
        steps[3] = archivePanel;
        steps[4] = runPanel;
        return steps;
    }

    //
    private void pmInit() {
        tabListener = new TabChangeListener();
        tab.addChangeListener(tabListener);
        generalPanel = new GeneralPanel();
        inputPanel = new InputPanel();
        contentPanel = new ContentPanel();
        archivePanel = new ArchivePanel();
        runPanel = new RunPanel();
        addInstructor(generalPanel);
        addInstructor(inputPanel);
        addInstructor(contentPanel);
        addInstructor(archivePanel);
        addInstructor(runPanel);
    }

    private void addInstructor(Instructor i) {
        InstructPanel ip = null;

        if (i instanceof JPanel) {
            ip = new InstructPanel();
            ip.addInstuctor(i);
            tab.add(ip, i.getTab());
        } else {
            throw new RuntimeException("Instructor not a JPanel: "
                                       + i.getTitle());
        }
    }

    private void jbInit() throws Exception {
        tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                              JTabbedPane.class.getName());
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        this.setLayout(layoutMain);
        this.add(tab, BorderLayout.CENTER);
    }

    private class TabChangeListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            if (!tab.getSelectedComponent().isShowing()) {
                int selectIndex = tab.getSelectedIndex();

                if (selectIndex > showIndex) {
                    tab.setSelectedIndex(showIndex);
                    for (int i = showIndex; i < selectIndex; i++) {
                        if (!next()) {
                            break;
                        }
                    }
                    showIndex = tab.getSelectedIndex();
                } else if (selectIndex < showIndex) {
                    showIndex = selectIndex;
                }
            }
            refresh();
        }

    }
}
