
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ResUtil;

// Standard imports
import java.io.File;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.ArrayList;

//
public class Main {
    static public boolean       debug = true;

    // not to be resourced
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.tool.archive.Res");     // nores

    //
    private static final String PARAM_HELP = "-help";                 // nores
    private static final String PARAM_EJB_ARCHIVE = "-ejbArchive";    // nores
    private static final String PARAM_EAR_ARCHIVE = "-earArchive";    // nores
    private static final String PARAM_JAR_ARCHIVE = "-jarArchive";    // nores
    private static final String PARAM_WEB_ARCHIVE = "-webArchive";    // nores
    private static final String PARAM_CLASSPATH = "-classpath";       // nores
    private static final String PARAM_FILES = "-files";               // nores
    private static final String PARAM_NO_ALT = "-noAlt";              // nores
    private static final String PARAM_LIB = "-lib";                   // nores

    public Main() {
        System.out.println("Kelp Archive Wizard " +
           ToolBoxInfo.getToolBoxVersion());
        System.out.println(ToolBoxInfo.getCopyright());
    }

    public static void main(String[] args) {
        Main main = new Main();

        main.init(args);
        System.exit(0);
    }

    private void init(String[] args) {
        ArchiveTool packager = null;
        File        newFile = null;
        JarPlan     plan = null;

        if (args.length < 1) {
            String newArchive = null;

            SwingUtil.setLookAndFeelToSystem();
            newArchive = ArchiveTool.runWizard();
            if (newArchive == null) {
                System.out.println("No archive created.");
            } else {
                System.out.println("Created archive: " + newArchive);
            }
        } else if (args[0].equalsIgnoreCase(PARAM_HELP)) {
            showUsage();
        } else {
            try {
                plan = parsePlanArgs(args);
                plan.setClassFiltering(true);
                packager = new ArchiveTool();
                newFile = packager.buildArchive(plan);
                System.out.println(ResUtil.format(res.getString("Created_file_0_"),
                                                  newFile));
            } catch (ArgumentException e) {
                System.out.println(ResUtil.format(res.getString("Error_0_"),
                                                  e.getMessage()));
                showUsage();
            } catch (ArchiveException e) {
                System.out.println(ResUtil.format(res.getString("Error_0_"),
                                                  e.getMessage()));
            }
        }
    }

    private JarPlan parsePlanArgs(String[] args) throws ArgumentException {
        JarPlan plan = null;

        if (args.length < 1) {
            throw new ArgumentException(res.getString("Too_few_arguments"));
        } else if (args[0].equalsIgnoreCase(PARAM_EJB_ARCHIVE)) {
            plan = parseEjbArgs(args);
        } else if (args[0].equalsIgnoreCase(PARAM_EAR_ARCHIVE)) {
            plan = parseEarArgs(args);
        } else if (args[0].equalsIgnoreCase(PARAM_JAR_ARCHIVE)) {
            plan = parseJarArgs(args);
        } else if (args[0].equalsIgnoreCase(PARAM_WEB_ARCHIVE)) {
            plan = parseWarArgs(args);
        } else {
            throw new ArgumentException(ResUtil.format(res.getString("First_argument_must"),
                                                       PARAM_WEB_ARCHIVE,
                                                       PARAM_JAR_ARCHIVE));
        }
        return plan;
    }

    private EjbPlan parseEjbArgs(String[] args) throws ArgumentException {
        EjbPlan ejbPlan = null;
        int     cursor = 1;

        if (args.length < 5) {
            throw new ArgumentException(res.getString("Too_few_arguments"));
        } else if (args.length >= 8) {
            throw new ArgumentException(ResUtil.format(res.getString("Too_many_arguments"),
                                                       PARAM_EJB_ARCHIVE));
        } else if (!args[1].equals(PARAM_CLASSPATH)) {
            throw new ArgumentException(ResUtil.format(res.getString("Missing_parameter_"),
                                                       PARAM_CLASSPATH));
        } else {
            try {
                Descriptor[] desc = new Descriptor[1];

                desc[0] = new Descriptor(true, Descriptor.EJB);
                ejbPlan = new EjbPlan();
                cursor = readClasses(ejbPlan, args);
                desc[0].setPath(args[cursor++]);
                desc[0].validate();
                ejbPlan.setDescriptors(desc);
                ejbPlan.setArchivePath(args[cursor]);
            } catch (ArchiveException e) {
                throw new ArgumentException(e.getMessage());
            }
        }
        return ejbPlan;
    }

    private EarPlan parseEarArgs(String[] args) throws ArgumentException {
        EarPlan         earPlan = null;
        StringTokenizer tokenizer = null;
        PathHandle ph  = null;
        int             cursor = 1;

        if (args.length < 4) {
            throw new ArgumentException(res.getString("Too_few_arguments"));
        } else if (args.length > 8) {
            throw new ArgumentException(ResUtil.format(res.getString("Too_many_arguments"),
                                                       PARAM_EAR_ARCHIVE));
        } else {
            try {
                earPlan = new EarPlan();
                {
                    String[] files = new String[0];

                    tokenizer = new StringTokenizer(args[cursor++], ":");
                    while (tokenizer.hasMoreTokens()) {
                        ph = PathHandle.createPathHandle(tokenizer.nextToken());
                        if (ph.hasExtension("war")) {
                          earPlan.addWebApplication(ph.getPath());
                        } else {
                          earPlan.addEnterpriseModule(ph.getPath());
                        }
                    }
                }
                cursor = readLibFiles(earPlan, args, cursor);
                if (args[cursor].equalsIgnoreCase(PARAM_NO_ALT)) {
                    earPlan.setAlt(false);
                    cursor++;
                }
                earPlan.setAppName(args[cursor++]);
                earPlan.setArchivePath(args[cursor]);
            } catch (ArchiveException e) {
                throw new ArgumentException(e.getMessage());
            }
        }
        return earPlan;
    }

    private JarPlan parseJarArgs(String[] args) throws ArgumentException {
        JarPlan jarPlan = null;
        int     cursor = 1;

        if (args.length < 4) {
            throw new ArgumentException(res.getString("Too_few_arguments"));
        } else if (args.length >= 7) {
            throw new ArgumentException(ResUtil.format(res.getString("Too_many_arguments"),
                                                       PARAM_JAR_ARCHIVE));
        } else if (!args[1].equals(PARAM_CLASSPATH)) {
            throw new ArgumentException(ResUtil.format(res.getString("Second_argument_must"),
                                                       PARAM_CLASSPATH));
        } else {
            try {
                jarPlan = new JarPlan();
                cursor = readClasses(jarPlan, args);
                jarPlan.setArchivePath(args[cursor]);
            } catch (ArchiveException e) {
                throw new ArgumentException(e.getMessage());
            }
        }
        return jarPlan;
    }

    private WarPlan parseWarArgs(String[] args) throws ArgumentException {
        WarPlan         warPlan = null;
        ArrayList       list = new ArrayList();
        StringTokenizer tokenizer = null;
        StringBuffer    buf = new StringBuffer();
        PathHandle      path = null;
        int             cursor = 1;

        if (args.length < 4) {
            throw new ArgumentException(res.getString("Too_few_arguments"));
        } else if (args.length > 10) {
            throw new ArgumentException(res.getString("Too_many_arguments1"));
        } else {
            try {
                Descriptor[] desc = new Descriptor[1];

                desc[0] = new Descriptor(true, Descriptor.WEB);
                warPlan = new WarPlan();
                cursor = readClasses(warPlan, args);
                warPlan.setContentRoot(args[cursor++]);

                // read content
                if (args[cursor].equalsIgnoreCase(PARAM_FILES)) {
                    String[] files = new String[0];

                    cursor++;
                    tokenizer = new StringTokenizer(args[cursor++], ":");
                    while (tokenizer.hasMoreTokens()) {
                        buf.setLength(0);
                        buf.append(warPlan.getContentRoot());
                        buf.append(File.separator);
                        buf.append(tokenizer.nextToken());
                        path = PathHandle.createPathHandle(buf.toString());
                        if (path.isFile()) {
                            list.add(path.getPath());
                        } else {
                            System.err.println("File not found: "
                                               + path.getPath());
                        }
                    }
                    list.trimToSize();
                    files = (String[]) list.toArray(files);
                    list.clear();
                    warPlan.setContentFiles(files);
                }
                cursor = readLibFiles(warPlan, args, cursor);
                desc[0].setPath(args[cursor++]);
                desc[0].validate();
                warPlan.setDescriptors(desc);
                warPlan.setArchivePath(args[cursor]);
            } catch (ArrayIndexOutOfBoundsException e) {
                throw new ArgumentException("Archiving syntax error");
            } catch (ArchiveException e) {
                throw new ArgumentException(e.getMessage());
            }
        }
        return warPlan;
    }

    private int readClasses(JarPlan plan,
                            String[] args) throws ArchiveException {
        StringTokenizer tokenizer = null;
        ArrayList       list = new ArrayList();
        StringBuffer    buf = new StringBuffer();
        PathHandle      path = null;
        int             cursor = 1;

        if (args[1].equalsIgnoreCase(PARAM_CLASSPATH)) {
            plan.setClassRoot(args[2]);
            cursor = 3;
            if (args[cursor].equalsIgnoreCase(PARAM_FILES)) {
                String[] files = new String[0];

                cursor++;
                tokenizer = new StringTokenizer(args[cursor++], ":");
                while (tokenizer.hasMoreTokens()) {
                    buf.setLength(0);
                    buf.append(args[2]);
                    buf.append(File.separator);
                    buf.append(tokenizer.nextToken());
                    path = PathHandle.createPathHandle(buf.toString());
                    if (path.isFile()) {
                        list.add(path.getPath());
                    } else {
                        System.err.println("File not found: "
                                           + path.getPath());
                    }
                }
                list.trimToSize();
                files = (String[]) list.toArray(files);
                list.clear();
                plan.setClassFiles(files);
            }
        }
        return cursor;
    }

    private int readLibFiles(JarPlan plan, String[] args,
                             int cursor) throws ArchiveException {
        StringTokenizer tokenizer = null;
        ArrayList       list = new ArrayList();

        if (args[cursor].equalsIgnoreCase(PARAM_LIB)) {
            cursor++;
            String[] files = new String[0];

            tokenizer = new StringTokenizer(args[cursor++], ":");
            while (tokenizer.hasMoreTokens()) {
                list.add(tokenizer.nextToken());
            }
            list.trimToSize();
            files = (String[]) list.toArray(files);
            list.clear();
            plan.setLibFiles(files);
        }
        return cursor;
    }

    private void showUsage() {
        System.out.println(new String());
        System.out.println(res.getString("Parameters_"));

        // EJB .jar archive
        System.out.print(' ');
        System.out.println(PARAM_EJB_ARCHIVE);
        System.out.print("     ");
        System.out.println(getClassUsage());
        System.out.print("     ");
        System.out.print("<ejb-jar.xml>");
        System.out.print(' ');
        System.out.println(res.getString("_archive_file_"));
        System.out.println(' ');

        // enterprise
        System.out.print(' ');
        System.out.println(PARAM_EAR_ARCHIVE);
        System.out.print("     ");
        System.out.println("moduleArchive1:moduleArchive2:...");
        System.out.print("     ");
        System.out.println("[" + PARAM_LIB + " jar1:jar2:...]");
        System.out.print("     ");
        System.out.println("[" + PARAM_NO_ALT + "]");
        System.out.print("     ");
        System.out.println("<application-name> <archive-fle>");
        System.out.println("");

        // jar archive
        System.out.print(' ');
        System.out.println(PARAM_JAR_ARCHIVE);
        System.out.print("     ");
        System.out.print(getClassUsage());
        System.out.print(' ');
        System.out.println(res.getString("_archive_file_"));
        System.out.println("");

        // web archive
        System.out.print(' ');
        System.out.println(PARAM_WEB_ARCHIVE);
        System.out.print("     ");
        System.out.println("[ " + getClassUsage() + " ]");
        System.out.print("     ");
        System.out.println("[" + PARAM_LIB + " jar1:jar2:...]");
        System.out.print("     ");
        System.out.print(res.getString("_content_root_"));
        System.out.print(' ');
        System.out.print('[');
        System.out.print(PARAM_FILES);
        System.out.print(' ');
        System.out.print("file1:file2...");
        System.out.println(']');
        System.out.print("     ");
        System.out.print(res.getString("_web_xml_"));
        System.out.print(' ');
        System.out.println(res.getString("_archive_file_"));
        System.out.println(' ');
    }

    private String getClassUsage() {
        StringBuffer buf = new StringBuffer();

        buf.append(PARAM_CLASSPATH);
        buf.append(' ');
        buf.append(res.getString("_classes_root_"));
        buf.append(' ');
        buf.append('[');
        buf.append(PARAM_FILES);
        buf.append(' ');
        buf.append("file1:file2...");
        buf.append(']');
        return buf.toString();
    }

    class ArgumentException extends Exception {
        public ArgumentException(String msg) {
            super(msg);
        }

    }
}
