
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.archive.Descriptor;
import org.enhydra.tool.archive.EarPlan;
import org.enhydra.tool.archive.EjbPlan;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.archive.ServicePlan;
import org.enhydra.tool.archive.WarPlan;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.ArchiveTool;
import org.enhydra.tool.archive.ManifestKeys;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ToolException;

// JDK
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.util.jar.JarInputStream;
import java.util.jar.JarEntry;
import java.util.jar.Manifest;

//
abstract public class ArchiveType implements ManifestKeys {
    private boolean mru = true;
    private JarPlan        plan = null;
    private ArchivePanel[] panels = new ArchivePanel[0];

    static public ArchiveType createType(JarPlan plan) {
        ArchiveType type = null;

        if (plan instanceof WarPlan) {
            type = new WarType();
            type.setPlan(plan);
        } else if (plan instanceof EjbPlan) {
            type = new EjbType();
            type.setPlan(plan);
        } else if (plan instanceof EarPlan) {
            type = new EarType();
            type.setPlan(plan);
        } else if (plan instanceof ServicePlan) {
            type = new ServiceType();
            type.setPlan(plan);
        } else {

            // default JarPlan
            type = new E3Type();
            type.setPlan(plan);
        }
        return type;
    }

    //
    static protected ArchiveType[] getAllTypes() {
        ArchiveType[] types = new ArchiveType[0];

        if (ToolBoxInfo.isEnhydra3()) {
            types = new ArchiveType[2];
            types[0] = new WarType();
            types[1] = new E3Type();
        } else {
            types = new ArchiveType[5];
            types[0] = new WarType();
            types[1] = new EjbType();
            types[2] = new EarType();
            types[3] = new ServiceType();
            types[4] = new E3Type();
        }
        return types;
    }

    //
    public void setMostRecentlyUsed(boolean b) {
      mru = b;
    }
    public boolean isMostRecentlyUsed() {
      return mru;
    }

    public String toString() {
        return getSelectionName();
    }

    public void readArchive(String archive) {
        JarInputStream      jar = null;
        BufferedInputStream buf = null;
        Manifest            manifest = null;
        JarEntry            entry = null;
        ArrayList           list = new ArrayList();
        String[]            entries = new String[0];
        PathHandle          path = PathHandle.createPathHandle(archive);

        if (path.isFile()) {
            try {
                getPlan().setArchivePath(path.getPath());
                buf =
                    new BufferedInputStream(new FileInputStream(path.getFile()));
                jar = new JarInputStream(buf);
                manifest = jar.getManifest();
                entry = jar.getNextJarEntry();
                while (entry != null) {
                    list.add(entry.getName());
                    entry = jar.getNextJarEntry();
                }
                list.trimToSize();
                entries = new String[list.size()];
                entries = (String[]) list.toArray(entries);
                list.clear();
                readArchive(manifest, entries);
                jar.close();
                buf.close();
            } catch (FileNotFoundException e) {
                e.printStackTrace(System.err);
            } catch (IOException e) {
                e.printStackTrace(System.err);
            } catch (ArchiveException e) {
                e.printStackTrace(System.err);
            }
        }
    }

    public JarPlan getPlan() throws ArchiveException {
        if (plan == null) {
            initPlan();
        }
        return plan;
    }

    //
    abstract protected String getSelectionName();
    abstract protected String getDescription();
    abstract protected String getExtension();
    abstract protected String getType();
    abstract protected void initPanels() throws ArchiveException;
    abstract protected void initPlan() throws ArchiveException;
    protected void setPlan(JarPlan p) {
        plan = p;
    }

    //
    protected String getClassIncludeRoot() {
        return new String();
    }

    protected boolean readArchive(Manifest manifest, String[] entries) {
        boolean      read = true;
        PathHandle   root = null;
        PathHandle   cursor = null;
        String[]     files = new String[0];
        Descriptor[] dd = new Descriptor[0];
        ArrayList    list = new ArrayList();

        if ((manifest == null)) {
            read = false;
        } else {
            String value = null;

            try {

                // classes
                value = manifest.getMainAttributes().getValue(CLASS_ROOT);
                root = PathHandle.createPathHandle(value);
                if (root.isDirectory()) {
                    getPlan().setClassRoot(root.getPath());
                    for (int i = 0; i < entries.length; i++) {
                        if (entries[i].toUpperCase().startsWith(getClassIncludeRoot())) {
                            cursor =
                                PathHandle.createPathHandle(root.getPath()
                                                            + File.separator
                                                            + entries[i].substring(getClassIncludeRoot().length()));
                            if (cursor.isFile()) {
                                list.add(cursor.getPath());
                            }
                        }
                    }
                    list.trimToSize();
                    files = new String[list.size()];
                    files = (String[]) list.toArray(files);
                    list.clear();
                    getPlan().setClassFiles(files);
                }

                // libraries (0 - 99)
                for (int i = 0; i < 100; i++) {
                    value = manifest.getMainAttributes().getValue(LIBRARY
                            + i);
                    cursor = PathHandle.createPathHandle(value);
                    if (cursor.isFile()) {
                        list.add(cursor.getPath());
                    }
                }
                files = new String[list.size()];
                files = (String[]) list.toArray(files);
                list.clear();
                getPlan().setLibFiles(files);

                // deployment descriptors
                dd = getPlan().getDescriptors();
                for (int i = 0; i < dd.length; i++) {
                    value =
                        manifest.getMainAttributes().getValue(dd[i].getType());
                    cursor = PathHandle.createPathHandle(value);
                    if (cursor.isFile()) {
                        dd[i].setPath(cursor.getPath());
                    }
                }
                getPlan().setDescriptors(dd);
            } catch (ArchiveException e) {
                e.printStackTrace(System.err);
            }
        }
        return read;
    }

    //
    protected String getWizardTitle() {
        StringBuffer buf = new StringBuffer();

        buf.append(getSelectionName());
        buf.append(" Wizard");
        return buf.toString();
    }

    //
    protected ArchivePanel[] getWizardPanels() throws ArchiveException {
        if (panels.length == 0) {
            initPanels();
        }
        return panels;
    }

    protected void setWizardPanels(ArchivePanel[] p) throws ArchiveException {
        panels = p;
        for (int i = 0; i < panels.length; i++) {
            panels[i].readPlan(getPlan());
        }
    }

    protected File build() throws ArchiveException {
        ArchiveTool tool = new ArchiveTool();

        return tool.buildArchive(getPlan());
    }

    protected void saveDefaults() {
        saveDefaultArchivePath();
    }

    private void saveDefaultArchivePath() {
        Properties props = null;

        try {
            props = ToolBoxInfo.loadProperties();
            props.setProperty(getType() + ".archive",
                              plan.getArchivePath());
            ToolBoxInfo.storeProperties(props);
        } catch (ToolException e) {
            e.printStackTrace(System.err);
            props = new Properties();
        }
    }

    protected String getDefaultArchivePath() {
        Properties   props = null;
        StringBuffer def = new StringBuffer();
        String       path = null;

        def.append(System.getProperty("user.dir"));
        def.append(File.separator);
        def.append("untitled.");
        def.append(getExtension());
        try {
            props = ToolBoxInfo.loadProperties();
        } catch (ToolException e) {
            e.printStackTrace(System.err);
            props = new Properties();
        }
        if (isMostRecentlyUsed()) {
          path = props.getProperty(getType() + ".archive", def.toString());
        } else {
          path = def.toString();
        }
        path = PathHandle.createPathString(path);
        return path;
    }

}
