
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.codegen.wizard;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.codegen.internal.AppOptionSet;
import org.enhydra.tool.codegen.GeneratorOption;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.ValidationException;
import org.enhydra.tool.codegen.ValidationUtil;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;

// JDK
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.*;
import java.io.File;
import java.util.Properties;
import java.util.ResourceBundle;

/**
 * Panel for entering the default options: project name, package
 * and destination.
 */
public class AppOptionPanel1 extends CodeGenPanel implements RootEditor {

    // not to be resourced
    private final String                  DEF_PACKAGE = "untitled";    // nores
    private final String                  DEF_PROJECT = "Untitled1";    // nores

    // All swing controls are public to allow for easy modification in IDE
    // implementations.

    /**
     * Label for project name
     */
    transient public JLabel               labelProjectName;

    /**
     * Text field for project name
     */
    transient public JTextField           textProjectName;

    /**
     * Label for package
     */
    transient public JLabel               labelPackage;

    /**
     * Text field for package
     */
    transient public JTextField           textPackage;

    /**
     * Panel for destination
     */
    transient public JPanel               panelDest;

    /**
     * Button for setting destination directory
     */
    transient public JButton              buttonSet;

    /**
     * Text field for destination
     */
    transient public JTextField           textDest;

    //
    transient private JPanel              panelFiller;
    transient private GridBagLayout       layoutMain;
    transient private GridBagLayout       layoutDest;
    transient private LocalButtonListener buttonListener;
    transient private TitledBorder        borderDest;
    transient private JLabel              labelClient;
    transient private JComboBox           comboClient;
    transient private JPanel              panelEnhydraRoot;
    transient private JTextField          textEnhydraRoot;
    transient private GridBagLayout       layoutEnhydraRoot;
    transient private Border              borderEnhydraRoot;
    transient private JButton             buttonEnhydraRoot;

    /**
     * Create default option panel 1 for entering the
     * following default options: project name, package and
     * destination.
     */
    public AppOptionPanel1() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // implements RootEditor
    public boolean isAllowRootEdit() {
      return textProjectName.isEnabled();
    }

    // implements RootEditor
    public void setAllowRootEdit(boolean allow) {
      textProjectName.setEnabled(allow);
      textDest.setEnabled(allow);
    }

    /**
     * Read project name, package and destination values from the
     * option set into the swing controls.
     *
     * @exception GeneratorException
     * Thrown if unable to update Swing controls with option set values.
     */
    public void readOptionSet() throws GeneratorException {
        GeneratorOption option = null;
        PathHandle      handle = null;

        // client type
        option = getOptionSet().lookup(AppOptionSet.CLIENT);
        comboClient.setSelectedItem(option.getValue());

        // project name
        option = getOptionSet().lookup(AppOptionSet.PROJECT);
        textProjectName.setText(option.getValue());

        // package
        option = getOptionSet().lookup(AppOptionSet.PACKAGE);
        textPackage.setText(option.getValue());

        // destination
        option = getOptionSet().lookup(AppOptionSet.ROOT);
        handle = PathHandle.createPathHandle(option.getValue());
        textDest.setText(handle.getPath());
    }

    /**
     * Write project name, package and destination values from the
     * swing controls into the option set.
     *
     * @exception GeneratorException
     * Thrown if unable to update option set from Swing control values.
     */
    public void writeOptionSet() throws GeneratorException {
        String value = new String();

        // client
        value = comboClient.getSelectedItem().toString().trim();
        getOptionSet().lookup(AppOptionSet.CLIENT).setValue(value);

        // name
        value = textProjectName.getText().trim();
        getOptionSet().lookup(AppOptionSet.PROJECT).setValue(value);

        // package
        value = textPackage.getText().trim();
        getOptionSet().lookup(AppOptionSet.PACKAGE).setValue(value);

        // destination
        value = textDest.getText().trim();
        getOptionSet().lookup(AppOptionSet.ROOT).setValue(value);
    }

    /**
     * Validate project name, package and destination values
     * in the swing controls.
     *
     * @exception ValidationException
     * Thrown if swing control values are not valid for the
     * current option set.
     */
    public void validateOptionSet() throws ValidationException {
        String value = new String();

        // project name
        value = textProjectName.getText().trim();
        if (!ValidationUtil.isJavaIdentifier(value)) {
            throw new ValidationException(res.getString("Project_directory"));
        }

        // package
        value = textPackage.getText().trim();
        if (!ValidationUtil.isJavaPackage(value)) {
            throw new ValidationException(res.getString("Package_is_not_valid"));
        }

        // destination
        value = textDest.getText().trim();
        if (!ValidationUtil.isParentDirectory(value)) {
            throw new ValidationException("Invalid Project Root");
        }
        if (ToolBoxInfo.isRootSettable()) {
            if (!ToolBoxInfo.isEnhydraRoot(ToolBoxInfo.getEnhydraRoot())) {
                throw new ValidationException("Invalid Application Server Root");
            }
        }
    }

    /**
     * Get the title to use on the current page.
     *
     * @return
     * A string to place at the top of a CodeGen wizard panel.
     */
    public String getPageTitle() {
        return res.getString("Client_type_and");
    }

    /**
     * Get the instructions for entering option values for the
     * current page.
     *
     * @return
     * A string to place below the page title.
     */
    public String getInstructions() {
        StringBuffer buf = new StringBuffer();

        buf.append(res.getString("Instruct1"));
        buf.append(res.getString("Instruct2"));
        buf.append(res.getString("Instruct3"));
        buf.append(res.getString("Instruct4"));
        return buf.toString();
    }

    //
    // PRIVATE
    //
    private void chooseEnhydraRoot() {
        File       choice = null;
        PathHandle path = null;
        Properties props = null;
        String     message = null;

        choice = SwingUtil.getDirectoryChoice(this,
                                              ToolBoxInfo.getEnhydraRoot(),
                                              "Select Application Server Root");
        path = PathHandle.createPathHandle(choice);
        if (path.isDirectory() && ToolBoxInfo.isEnhydraRoot(path.getPath())) {
            try {
                props = ToolBoxInfo.loadProperties();
                props.setProperty(ToolBoxInfo.ENHYDRA_ROOT, path.getPath());
                ToolBoxInfo.storeProperties(props);
                textEnhydraRoot.setText(ToolBoxInfo.getEnhydraRoot());
                textEnhydraRoot.setToolTipText(textEnhydraRoot.getText());
            } catch (ToolException e) {
                message = e.getMessage();
            }
        } else {
            message = "Invalid Application Server Root";
        }
        if (message != null) {
          JOptionPane.showMessageDialog(this, message, "Kelp Error",
                                        JOptionPane.ERROR_MESSAGE);
        }
    }

    /**
     * Method declaration
     *
     *
     * @exception Exception
     *
     * @see
     */
    private void jbInit() throws Exception {
        textProjectName =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        labelPackage = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                  JLabel.class.getName());
        textPackage =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        panelDest = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        layoutDest =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        buttonSet = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                JButton.class.getName());
        textDest = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        labelProjectName =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        labelClient = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                 JLabel.class.getName());
        comboClient =
            (JComboBox) Beans.instantiate(getClass().getClassLoader(),
                                          JComboBox.class.getName());
        panelEnhydraRoot =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        textEnhydraRoot =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonEnhydraRoot =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutEnhydraRoot =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        borderEnhydraRoot =
            BorderFactory.createTitledBorder("Application Server Root");
        labelProjectName.setMaximumSize(new Dimension(120, 17));
        labelProjectName.setMinimumSize(new Dimension(120, 17));
        labelProjectName.setPreferredSize(new Dimension(120, 17));
        labelProjectName.setText(res.getString("Project_directory1"));
        labelPackage.setText(res.getString("Package_"));
        buttonSet.setText(res.getString("Set_"));
        panelDest.setBorder(BorderFactory.createTitledBorder("Project Root"));
        panelDest.setLayout(layoutDest);
        labelClient.setText(res.getString("Client_type_"));
        panelDest.add(textDest,
                      new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(4, 5, 3, 5), 0, 0));
        panelDest.add(buttonSet,
                      new GridBagConstraints(1, 0, 1, 2, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(4, 5, 3, 10), 0, 0));
        textEnhydraRoot.setEnabled(false);
        buttonEnhydraRoot.setText(res.getString("Set_"));
        panelEnhydraRoot.setLayout(layoutEnhydraRoot);
        panelEnhydraRoot.setBorder(borderEnhydraRoot);
        panelEnhydraRoot.add(textEnhydraRoot,
                             new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                                    GridBagConstraints.WEST,
                                                    GridBagConstraints.HORIZONTAL,
                                                    new Insets(4, 5, 3, 5),
                                                    0, 0));
        panelEnhydraRoot.add(buttonEnhydraRoot,
                             new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                                                    GridBagConstraints.CENTER,
                                                    GridBagConstraints.NONE,
                                                    new Insets(4, 5, 3, 10),
                                                    0, 0));
        this.setLayout(layoutMain);
        this.add(labelProjectName, new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, 5, 3, 5), 50, 0));
        this.add(labelPackage, new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, 5, 3, 5), 50, 0));
        this.add(textProjectName, new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, 5, 3, 5), 150, 0));
        this.add(panelDest,
                 new GridBagConstraints(0, 3, 2, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(3, 5, 3, 5), 0, 0));
        this.add(textPackage, new GridBagConstraints(1, 1, 1, 1, 0.1, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, 5, 3, 5), 150, 0));
        this.add(panelFiller,
                 new GridBagConstraints(0, 5, 2, 1, 0.4, 0.4,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), 0, 0));
        this.add(labelClient, new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 3, 5), 50, 0));
        this.add(comboClient, new GridBagConstraints(1, 2, 1, 1, 0.1, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 3, 5), 50, 0));
        this.add(panelEnhydraRoot,
                 new GridBagConstraints(0, 4, 4, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(3, 5, 5, 5), 0, 0));
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void pmInit() {
        String[] docTypes = new String[0];

        buttonListener = new LocalButtonListener();
        buttonSet.addActionListener(buttonListener);
        textDest.setText(new String());
        textPackage.setText(DEF_PACKAGE);
        textProjectName.setText(DEF_PROJECT);
        comboClient.removeAllItems();
        docTypes = ToolBoxInfo.getSupportedDocTypes();
        for (int i = 0; i < docTypes.length; i++) {
            comboClient.addItem(docTypes[i].toUpperCase());
        }
        textEnhydraRoot.setText(ToolBoxInfo.getEnhydraRoot());
        textEnhydraRoot.setToolTipText(textEnhydraRoot.getText());
        textEnhydraRoot.setEnabled(false);
        if (ToolBoxInfo.isRootSettable()) {
            buttonListener = new LocalButtonListener();
            buttonEnhydraRoot.addActionListener(buttonListener);
            buttonEnhydraRoot.setEnabled(true);
        } else {
            buttonEnhydraRoot.setVisible(false);
            panelEnhydraRoot.remove(buttonEnhydraRoot);
        }
    }

    private void browseForFile() {
        File destDir = null;

        destDir =
            SwingUtil.getDirectoryChoice(this, textDest.getText(),
                                         res.getString("Choose_the_root_path"));
        if (destDir != null) {
            textDest.setText(PathHandle.createPathString(destDir));
        }
    }

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonSet) {
                browseForFile();
            } else if (source == buttonEnhydraRoot) {
                chooseEnhydraRoot();
            }
        }

    }
}
