
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.common;

import java.util.ResourceBundle;

/**
 * The replacement class defines a search and replace operation
 * for updating source code that was copied from a template file.
 * Replacement object are used by the predefined generators
 * to insert things like application and package names into
 * java source files.
 */
public class Replacement {
    static ResourceBundle       res =
        ResourceBundle.getBundle ("org.enhydra.tool.common.Res");       // nores

    //

    private String              find = new String ();
    private String[]            replaceWith = new String[0];

    /**
     * Define a replacement operation using an replacement array.
     *
     * @param find
     * String to look for. These strings should be deliniated
     * with '@' when not used for a file path. When
     * the replacement does reference a file path, the find
     * value should be prefixed with 'at_' and suffixed
     * with '_at'.
     * @param replaceWith
     * One or more strings to use as a replacement for the find string. If the array
     * contains more than one string, a new line is inserted for each element of the
     * replaceWith array.
     *
     * @throws GenerationException
     * Thrown if either parameter is null, or the find element is an empty string
     * or if the replaceWith array has a length of zero.
     */
    public Replacement (String inFind,
                        String[] inReplaceWith) throws ToolException {
        construct (inFind, inReplaceWith);
    }

    /**
     * Define a replacement operation using a single replacement string.
     *
     * @param find
     * String to look for. These strings should be deliniated
     * with '@' when not used for a file path. When
     * the replacement does reference a file path, the find
     * value should be prefixed with 'at_' and suffixed
     * with '_at'.
     * @param replaceWith
     * String to use as a replacement for the find string.
     *
     * @throws GenerationException
     * Thrown if either parameter is null or either element references
     * an empty string.
     */
    public Replacement (String inFind,
                        String inReplaceWith) throws ToolException {
        String[]        rw = new String[1];

        if (inReplaceWith == null) {
            inReplaceWith = new String ();
        }

        rw[0] = inReplaceWith;

        construct (inFind, rw);
    }

    /**
     * Get the string to search for.
     *
     * @return
     * The search for string.
     *
     */
    public String getFind () {
        return find;
    }

    /**
     * Get replacement text that can be a one or more stings. More
     * than one string indicates that the token should be replaced
     * with multiple lines.
     *
     * @return
     * An array of replacement strings.
     */
    public String[] getReplaceWith () {
        return replaceWith;
    }

    /**
     * Set the replaceWith to an array.
     *
     * @param with
     * One or more strings to use as a replacement for the find string. If
     * the array contains more than one string, a new line is inserted for
     * each element of the replaceWith array.
     */
    public void setReplaceWith (String[] with) {
        replaceWith = with;
    }

    /**
     * Set the replaceWith to a single string value.
     *
     * @param with
     * String to use as a replacement for the find string.
     */
    public void setReplaceWith (String with) {
        String[]        array = new String[1];

        array[0] = with;
        replaceWith = array;
    }

    //
    //

    private void construct (String inFind,
                            String[] inReplaceWith) throws ToolException {
        if (inFind == null || inFind.trim ().length () == 0) {
            throw new ToolException (res.getString ("Unable_to_create"));
        }

        if (inReplaceWith == null || inReplaceWith.length == 0) {
            replaceWith = new String[1];
            replaceWith[0] = new String ();
        }

        find = inFind;
        replaceWith = inReplaceWith;
    }

}

