
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen;

// ToolBox imports
import org.enhydra.tool.codegen.wizard.CodeGenPanel;

// Standard imports
import java.io.File;
import java.util.Properties;

/**
 * Interface for defining generators. CodeGen can use any generator
 * that implements this interface, is in the class path and has an
 * entry in the codegen.properties file.
 */
public interface Generator {

    /**
     * Set CodeGen customization properties. The CodeGen
     * class calls this method after it instantiates a
     * generator class.
     *
     * @param properties
     * Properties passed from the CodeGen class.
     */
    abstract public void setProperties(Properties properties)
            throws GeneratorException;

    /**
     * Get properties used to customize CodeGen. This may contain
     * information on the template directory location and
     * persisted generation options.
     *
     * @return
     * Properties passed from the CodeGen class.
     */
    abstract public Properties getProperties();

    /**
     * Get short name used in command line. May also be used
     * to locate template directory.
     *
     * @return
     * Command line identifier.
     */
    abstract public String getCommandName();

    /**
     * Name to display in command line messages and in the wizard.
     *
     * @return
     * Display name for generator.
     */
    abstract public String getDisplayName();

    /**
     * Name to display in command line messages and in the wizard.
     *
     * @return
     * Display name for generator.
     */
    abstract public String getWizardTitle();


    /**
     * Get text that breifly explains the application that
     * the generator creates.
     *
     *
     * @return
     * Descriptive text to display in a wizard or in a command
     * line help response.
     *
     */
    abstract public String getDescription();

    /**
     * Get an array of generator options that can
     * be set through command line options or through
     * wizard panels.
     *
     * @return
     * An OptionSet that contains an array of GeneratorOptions.
     *
     */
    abstract public OptionSet getOptionSet();

    /**
     * Get swing panels for setting generator options.
     * The panels can be used within a standalone
     * wizard or incorporated into the JBuilder wizard
     * framework.
     *
     * @return
     * An array of one or more JPanel based panels.
     * Return null to specify that the wizard only
     * supports a command line interface.
     */
    abstract public CodeGenPanel[] getWizardPanels()
            throws GeneratorException;

    /**
     * Try the generating the application and return a list
     *
     * @param to
     * The destination directory to generate source files into.
     *
     * @param overwrite
     * If true, the generator will overwrite existing files without
     * warning. Otherwise, any attempt to overwrite results in an
     * exception.
     *
     * @return
     * An array of the generated files.
     *
     * @throws GeneratorException
     * Thrown if all the files could not be generated without error.
     *
     */
    abstract public File[] generate() throws GeneratorException;


    /**
     * Get steps required to build the project with an IDE addin.
     *
     * @return
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project within an IDE.
     */
    abstract public String[] getAddinSteps();

    /**
     * Set the steps required to build the project from
     * within an IDE.
     *
     * @param steps
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project within an IDE.
     */
    abstract public void setAddinSteps(String[] steps);

    /**
     * Get steps required to build the project from
     * a unix shell.
     *
     * @return
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project using make files.
     */
    abstract public String[] getShellSteps();

    /**
     * Set the steps required to build the project from
     * a unix shell.
     *
     * @param steps
     * An array of strings, each string will be inserted into
     * a readme.html file as numbered step for building the
     * project from a unix shell.
     */
    abstract public void setShellSteps(String[] steps);

    abstract public void setEcho(boolean b);
    abstract boolean isEcho();
    abstract public void setSwing(boolean b);
    abstract boolean isSwing();

}
